<?php

namespace InspireLabs\WooInpost;

use Exception;
use InspireLabs\WooInpost\admin\Alerts;
use InspireLabs\WooInpost\admin\EasyPack_Settings_General;
use InspireLabs\WooInpost\EmailFilters\NewOrderEmail;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_C2C;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_COD;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_Local_Express;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_Local_Express_COD;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_Local_Standard;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_Local_Standard_COD;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_LSE;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_LSE_COD;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_Palette;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier_Palette_COD;
use InspireLabs\WooInpost\shipping\Easypack_Shipping_Rates;
use InspireLabs\WooInpost\shipping\EasyPack_Shippng_Parcel_Machines;
use InspireLabs\WooInpost\shipping\EasyPack_Shippng_Parcel_Machines_COD;
use InspireLabs\WooInpost\shipx\services\courier_pickup\ShipX_Courier_Pickup_Service;
use InspireLabs\WooInpost\shipx\services\organization\ShipX_Organization_Service;
use InspireLabs\WooInpost\shipx\services\shipment\ShipX_Shipment_Price_Calculator_Service;
use InspireLabs\WooInpost\shipx\services\shipment\ShipX_Shipment_Service;
use InspireLabs\WooInpost\shipx\services\shipment\ShipX_Shipment_Status_Service;
use WC_Order;
use WC_Shipping_Method;

class EasyPack extends inspire_Plugin4 {

	const LABELS_DIRECTORY
		= __DIR__ . DIRECTORY_SEPARATOR . 'web' . DIRECTORY_SEPARATOR
		  . 'labels';

	const CLASSES_DIRECTORY = __DIR__ . DIRECTORY_SEPARATOR . 'classes';

	const APP_REFIX = 'woo_inpost';

	public static $instance;

	public static $text_domain = 'woocommerce-inpost';

	protected $_pluginNamespace = "woo-inpost";

	/**
	 * @var WC_Shipping_Method[]
	 */
	protected $shipping_methods = [];

	protected $settings;

	/**
	 * @return string
	 */
	public static function getLabelsUri() {
		return plugins_url() . '/woo-inpost/web/labels/';
	}

	public function __construct() {
		parent::__construct();
		add_action( 'plugins_loaded', [ $this, 'init_easypack' ], 100 );
	}

	public static function EasyPack() {
		if ( self::$instance === null ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function init_easypack() {
		$this->init_alerts();

		$alerts = new Alerts();

		add_filter( 'woocommerce_get_settings_pages',
			[ $this, 'woocommerce_get_settings_pages' ] );
		add_action( 'admin_enqueue_scripts',
			[ $this, 'enqueue_admin_scripts' ], 75 );
		add_action( 'admin_footer', [ $this, 'admin_footer' ], 75 );

		try {
			( new Easypack_Shipping_Rates() )->init();
			$this->init_shipping_methods();

			add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ],
				75 );
			add_action( 'wp_footer', [ $this, 'frontFooter' ], 75 );
			add_filter( 'woocommerce_shipping_methods',
				[ $this, 'woocommerce_shipping_methods' ] );
			add_filter( 'woocommerce_get_order_item_totals',
				[ $this, 'show_parcel_machine_in_order_details' ], 2, 100 );
			add_action( 'wp_footer', [ $this, 'frontFooter' ], 75 );
			$this->init_email_filters();
		} catch ( Exception $exception ) {
			error_log( $exception->getMessage() );
		}
	}

	private function init_email_filters() {
		( new NewOrderEmail() )->init();
	}

	private function init_alerts() {
		$alerts = new Alerts();
	}

	/**
	 * @param array $items
	 *
	 * @param WC_Order $wcOrder
	 *
	 * @return array
	 */
	public function show_parcel_machine_in_order_details( $items, $wcOrder ) {
		$parcel_desc
			= html_entity_decode( $wcOrder->get_meta( '_parcel_machine_desc' ) );

		if ( isset( $items['shipping'] ) && ! empty( $parcel_desc ) ) {
			$items['shipping']['value']
				.= '<br>'
				   . sprintf( __( ' Selected parcel machine' )
				              . ': <br><span class="italic">%s', $parcel_desc )
				   . '</span>';
		}

		return $items;
	}

	private function include_shipment_models() {
		include( 'classes/shipx/models/shipment/class_shipment.php' );
		include( 'classes/shipx/models/shipment/class_shipment_cod.php' );
		include( 'classes/shipx/models/shipment/class_shipment_contstants.php' );
		include( 'classes/shipx/models/shipment/class_shipment_custom_attributes.php' );
		include( 'classes/shipx/models/shipment/class_shipment_parcel.php' );
		include( 'classes/shipx/models/shipment/class_shipment_parcel_dimensions.php' );
		include( 'classes/shipx/models/shipment/class_shipment_parcel_weight.php' );
		include( 'classes/shipx/models/shipment/class_shipment_receiver.php' );
		include( 'classes/shipx/models/shipment/class_shipment_receiver_address.php' );
		include( 'classes/shipx/models/shipment/class_shipment_sender.php' );
		include( 'classes/shipx/models/shipment/class_shipment_sender_address.php' );
		include( 'classes/shipx/models/shipment/class_shipment_internal_data.php' );
		include( 'classes/shipx/models/shipment/class_shipment_insurance.php' );
		include( 'classes/shipx/models/shipment/class_shipment_dispatch_status.php' );

		include( 'classes/shipx/models/organization/class_organization.php' );
		include( 'classes/shipx/models/organization/services/class_service.php' );
		include( 'classes/shipx/models/shipment_cost/class_shipment_cost.php' );
		include( 'classes/shipx/models/shipment_cost/class_shipment_cost_error.php' );
		include( 'classes/shipx/models/organization/services/class_additional_service.php' );
		include( 'classes/shipx/models/courier_pickup/class_dispatch_order.php' );
		include( 'classes/shipx/models/courier_pickup/class_dispatch_order_point.php' );
		include( 'classes/shipx/models/courier_pickup/class_dispatch_order_point_address.php' );
		include( 'classes/shipx/models/courier_pickup/class_dispatch_order_shipment.php' );
		include( 'classes/shipx/models/courier_pickup/class_dispatch_order_internal_data.php' );

		include( 'classes/shipx/services/organization/class_organization_service.php' );
		include( 'classes/shipx/services/shipment/class-shipment-service.php' );
		include( 'classes/shipx/services/shipment/class_shipment_status_service.php' );
		include( 'classes/shipx/services/shipment/class_shipment_price_caclulator_service.php' );
		include( 'classes/shipx/services/courier_pickup/class_courier_pickup_service.php' );


	}

	public function frontFooter() {
		echo '<script async src="https://geowidget.easypack24.net/js/sdk-for-javascript.js"></script>';

	}

	public function init_shipping_methods() {
		$organization_service = EasyPack::EasyPack()
		                                ->get_organization_service();
		$organization
		                      = $organization_service->query_organisation();

		if ( ! is_object( $organization ) ) {
			throw new Exception( 'Query organisation failed' );
		}

		$services_for_organization = $organization->getServices();

		foreach ( $services_for_organization as $service ) {
			$servicesAllowed[] = $service->getId();
		}

		if ( in_array(
			EasyPack_Shippng_Parcel_Machines::SERVICE_ID,
			$servicesAllowed )
		) {

			$easyPack_Shippng_Parcel_Machines
				                      = new EasyPack_Shippng_Parcel_Machines();
			$this->shipping_methods[] = $easyPack_Shippng_Parcel_Machines;
		}


		if ( in_array(
			EasyPack_Shippng_Parcel_Machines_COD::SERVICE_ID,
			$servicesAllowed )
		) {
			$easyPack_Shippng_Parcel_Machines_Cod
				= new EasyPack_Shippng_Parcel_Machines_COD();
			$this->shipping_methods[]
				= $easyPack_Shippng_Parcel_Machines_Cod;


			if ( in_array(
				EasyPack_Shipping_Method_Courier_Local_Standard::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_local_standard
					= new EasyPack_Shipping_Method_Courier_Local_Standard();
				$this->shipping_methods[]
					= $shipping_Method_Courier_local_standard;
			}


			if ( in_array(
				EasyPack_Shipping_Method_Courier_LSE_COD::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_LSE_COD
					= new EasyPack_Shipping_Method_Courier_LSE_COD();
				$this->shipping_methods[]
					= $shipping_Method_Courier_LSE_COD;
			}


			if ( in_array(
				EasyPack_Shipping_Method_Courier_COD::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_COD
					                      = new EasyPack_Shipping_Method_Courier_COD();
				$this->shipping_methods[] = $shipping_Method_Courier_COD;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier
					                      = new EasyPack_Shipping_Method_Courier();
				$this->shipping_methods[] = $shipping_Method_Courier;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier_LSE::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_LSE
					                      = new EasyPack_Shipping_Method_Courier_LSE();
				$this->shipping_methods[] = $shipping_Method_Courier_LSE;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier_LSE_COD::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_LSE_COD
					= new EasyPack_Shipping_Method_Courier_LSE_COD();
				$this->shipping_methods[]
					= $shipping_Method_Courier_LSE_COD;
			}


			if ( in_array(
				EasyPack_Shipping_Method_Courier_Local_Standard_COD::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_local_standard_cod
					= new EasyPack_Shipping_Method_Courier_Local_Standard_COD();
				$this->shipping_methods[]
					= $shipping_Method_Courier_local_standard_cod;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier_Local_Express::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_local_express
					= new EasyPack_Shipping_Method_Courier_Local_Express();
				$this->shipping_methods[]
					= $shipping_Method_Courier_local_express;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier_Local_Express_COD::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_local_express_cod
					= new EasyPack_Shipping_Method_Courier_Local_Express_COD();
				$this->shipping_methods[]
					= $shipping_Method_Courier_local_express_cod;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier_Palette::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_Palette
					= new EasyPack_Shipping_Method_Courier_Palette();
				$this->shipping_methods[]
					= $shipping_Method_Courier_Palette;
			}

			if ( in_array(
				EasyPack_Shipping_Method_Courier_Palette_COD::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_Palette_Cod
					= new EasyPack_Shipping_Method_Courier_Palette_COD();
				$this->shipping_methods[]
					= $shipping_Method_Courier_Palette_Cod;
			}


			if ( in_array(
				EasyPack_Shipping_Method_Courier_C2C::SERVICE_ID,
				$servicesAllowed )
			) {
				$shipping_Method_Courier_c2c
					= new EasyPack_Shipping_Method_Courier_C2C();
				$this->shipping_methods[]
					= $shipping_Method_Courier_c2c;
			}
		}
	}

	public
	function woocommerce_shipping_methods(
		$methods
	) {
		//die('&&&&&&&');
		foreach ( $this->shipping_methods as $shipping_method ) {
			$methods[ $shipping_method->id ] = get_class( $shipping_method );
		}

		return $methods;
	}

	public
	function woocommerce_get_settings_pages(
		$woocommerce_settings
	) {

		//$settings = include('classes/admin/class-settings-general.php');
		new EasyPack_Settings_General;


		//$woocommerce_settings[] = $settings;

		return $woocommerce_settings;
	}

	public
	function get_package_sizes() {
		return [
			'small'  => __( 'A 8 x 38 x 64 cm', EasyPack::$text_domain ),
			'medium' => __( 'B 19 x 38 x 64 cm', EasyPack::$text_domain ),
			'large'  => __( 'C 41 x 38 x 64 cm', EasyPack::$text_domain ),
		];
	}

	public
	function get_package_sizes_display() {
		return [
			'small'  => __( 'A', EasyPack::$text_domain ),
			'medium' => __( 'B', EasyPack::$text_domain ),
			'large'  => __( 'C', EasyPack::$text_domain ),
		];
	}

	public
	function get_package_weights_parcel_machines() {
		return [
			'1'  => __( '1 kg', EasyPack::$text_domain ),
			'2'  => __( '2 kg', EasyPack::$text_domain ),
			'5'  => __( '5 kg', EasyPack::$text_domain ),
			'10' => __( '10 kg', EasyPack::$text_domain ),
			'15' => __( '15 kg', EasyPack::$text_domain ),
			'20' => __( '20 kg', EasyPack::$text_domain ),
		];
	}

	public
	function get_package_weights_courier() {
		return [
			'1'  => __( '1 kg', EasyPack::$text_domain ),
			'2'  => __( '2 kg', EasyPack::$text_domain ),
			'5'  => __( '5 kg', EasyPack::$text_domain ),
			'10' => __( '10 kg', EasyPack::$text_domain ),
			'15' => __( '15 kg', EasyPack::$text_domain ),
			'20' => __( '20 kg', EasyPack::$text_domain ),
			'25' => __( '25 kg', EasyPack::$text_domain ),
		];
	}

	public
	function loadPluginTextDomain() {
		parent::loadPluginTextDomain();
		$ret = load_plugin_textdomain( EasyPack::$text_domain, false,
			'woocommerce-inpost/languages' );
	}

	public
	static function getTextDomain() {
		return EasyPack::$text_domain;
	}


	function getTemplatePathFull() {
		return implode( '/', [ $this->_pluginPath, $this->getTemplatePath() ] );
	}

	public
	function enqueue_scripts() {
		wp_enqueue_style( 'easypack-front',
			$this->getPluginCss() . 'front.css' );
		wp_enqueue_style( 'geowidget-4.5-css',
			EasyPack_API()->getGeoWidgetApiCssUrl() );
		wp_enqueue_script( 'easypack-front-js',
			$this->getPluginJs() . 'front.js', [ 'jquery' ] );

	}

	function enqueue_admin_scripts() {

		wp_enqueue_style( 'easypack-admin',
			$this->getPluginCss() . 'admin.css' );
		wp_enqueue_style( 'easypack-admin-modal',
			$this->getPluginCss() . 'modal.css' );
		wp_enqueue_script( 'easypack-admin',
			$this->getPluginJs() . 'admin.js', [ 'jquery' ] );
		wp_enqueue_media();//logo upload dependency
		wp_enqueue_script( 'easypack-admin-modal',
			$this->getPluginJs() . 'modal.js', [ 'jquery' ] );
		wp_enqueue_script( 'easypack-admin-geowidget-settings',
			$this->getPluginJs() . 'admin-geowidget-settings.js',
			[ 'jquery' ] );


		wp_enqueue_style( 'geowidget-4.5-css',
			EasyPack_API()->getGeoWidgetApiCssUrl() );
	}

	function admin_footer() {
		echo '<script async src="https://geowidget.easypack24.net/js/sdk-for-javascript.js"></script>';
	}

	/**
	 * @return ShipX_Shipment_Service
	 */
	public
	function get_shipment_service() {
		return new ShipX_Shipment_Service();
	}

	/**
	 * @return ShipX_Organization_Service
	 */
	public
	function get_organization_service() {
		return new ShipX_Organization_Service();
	}

	/**
	 * @return ShipX_Shipment_Price_Calculator_Service
	 */
	public
	function get_shipment_price_calculator_service() {
		return new ShipX_Shipment_Price_Calculator_Service();
	}

	/**
	 * @return ShipX_Courier_Pickup_Service
	 */
	public
	function get_courier_pickup_service() {
		return new ShipX_Courier_Pickup_Service();
	}

	/**
	 * @return ShipX_Shipment_Status_Service
	 */
	public
	function get_shipment_status_service() {
		return new ShipX_Shipment_Status_Service();
	}

	/**
	 * action_links function.
	 *
	 * @access public
	 *
	 * @param mixed $links
	 *
	 * @return void
	 */

	public
	function linksFilter(
		$links
	) {
		$plugin_links = [
			'<a href="'
			. admin_url( 'admin.php?page=wc-settings&tab=easypack_general' )
			. '">'
			. __( 'Settings', EasyPack::$text_domain ) . '</a>',
			'<a href="https://wordpress.org/plugins/woo-inpost/">'
			. __( 'Documentation', EasyPack::$text_domain ) . '</a>',
			'<a href="https://wordpress.org/support/plugin/woo-inpost">'
			. __( 'Support', EasyPack::$text_domain ) . '</a>',
		];

		return array_merge( $plugin_links, $links );
	}
}




